@@ -31,24 +31,25 @@
 #include "iplugin.h"
 
 #include <QCoreApplication>
-#include <QEventLoop>
 #include <QDateTime>
+#include <QDebug>
 #include <QDir>
+#include <QEventLoop>
 #include <QFile>
 #include <QLibrary>
 #include <QLibraryInfo>
 #include <QMetaProperty>
 #include <QSettings>
-#include <QTextStream>
-#include <QTime>
-#include <QWriteLocker>
-#include <QDebug>
-#include <QTimer>
 #include <QSysInfo>
+#include <QTextStream>
+#include <QTimer>
+#include <QWriteLocker>
 
 #include <utils/algorithm.h>
+#include <utils/benchmarker.h>
 #include <utils/executeondestruction.h>
 #include <utils/hostosinfo.h>
+#include <utils/mimetypes/mimedatabase.h>
 #include <utils/qtcassert.h>
 #include <utils/synchronousprocess.h>
 
@@ -59,7 +60,7 @@
 
 #include <functional>
 
-Q_LOGGING_CATEGORY(pluginLog, "qtc.extensionsystem")
+Q_LOGGING_CATEGORY(pluginLog, "qtc.extensionsystem", QtWarningMsg)
 
 const char C_IGNORED_PLUGINS[] = "Plugins/Ignored";
 const char C_FORCEENABLED_PLUGINS[] = "Plugins/ForceEnabled";
@@ -105,7 +106,7 @@ enum { debugLeaks = 0 };
     loading.
     \code
         // 'plugins' and subdirs will be searched for plugins
-        PluginManager::setPluginPaths(QStringList() << "plugins");
+        PluginManager::setPluginPaths(QStringList("plugins"));
         PluginManager::loadPlugins(); // try to load all the plugins
     \endcode
     Additionally, it is possible to directly access the plugin specifications
@@ -115,8 +116,8 @@ enum { debugLeaks = 0 };
     \section1 Object Pool
     Plugins (and everybody else) can add objects to a common 'pool' that is located in
     the plugin manager. Objects in the pool must derive from QObject, there are no other
-    prerequisites. All objects of a specified type can be retrieved from the object pool
-    via the getObjects() and getObject() functions.
+    prerequisites. Objects can be retrieved from the object pool via the getObject()
+    and getObjectByName() functions.
 
     Whenever the state of the object pool changes a corresponding signal is emitted by the plugin manager.
 
@@ -130,8 +131,8 @@ enum { debugLeaks = 0 };
         MyMimeTypeHandler *handler = new MyMimeTypeHandler();
         PluginManager::instance()->addObject(handler);
         // In plugin A:
-        QList<MimeTypeHandler *> mimeHandlers =
-            PluginManager::getObjects<MimeTypeHandler>();
+        MimeTypeHandler *mimeHandler =
+            PluginManager::getObject<MimeTypeHandler>();
     \endcode
 
 
@@ -247,36 +248,15 @@ enum { debugLeaks = 0 };
     \sa addObject()
 */
 
-/*!
-    \fn QList<T *> PluginManager::getObjects()
 
-    Retrieves all objects of a given type from the object pool.
-
-    This function uses \c qobject_cast to determine the type of an object.
-
-    \sa addObject()
-*/
-
-/*!
-    \fn QList<T *> PluginManager::getObjects(Predicate predicate)
-
-    Retrieves all objects of a given type from the object pool that
-    match the \a predicate.
-
-    This function uses \c qobject_cast to determine the type of an object.
-    The predicate should be a unary function taking a T* parameter and
-    returning a bool.
-
-    \sa addObject()
-*/
-
-
-using namespace ExtensionSystem;
-using namespace ExtensionSystem::Internal;
 using namespace Utils;
 
-static Internal::PluginManagerPrivate *d = 0;
-static PluginManager *m_instance = 0;
+namespace ExtensionSystem {
+
+using namespace Internal;
+
+static Internal::PluginManagerPrivate *d = nullptr;
+static PluginManager *m_instance = nullptr;
 
 /*!
     Gets the unique plugin manager instance.
@@ -301,7 +281,7 @@ PluginManager::PluginManager()
 PluginManager::~PluginManager()
 {
     delete d;
-    d = 0;
+    d = nullptr;
 }
 
 /*!
@@ -315,7 +295,7 @@ PluginManager::~PluginManager()
 
     \sa PluginManager::removeObject()
     \sa PluginManager::getObject()
-    \sa PluginManager::getObjects()
+    \sa PluginManager::getObjectByName()
 */
 void PluginManager::addObject(QObject *obj)
 {
@@ -337,9 +317,8 @@ void PluginManager::removeObject(QObject *obj)
     Usually, clients do not need to call this function.
 
     \sa PluginManager::getObject()
-    \sa PluginManager::getObjects()
 */
-QList<QObject *> PluginManager::allObjects()
+QVector<QObject *> PluginManager::allObjects()
 {
     return d->allObjects;
 }
@@ -359,7 +338,7 @@ QReadWriteLock *PluginManager::listLock()
 */
 void PluginManager::loadPlugins()
 {
-    return d->loadPlugins();
+    d->loadPlugins();
 }
 
 /*!
@@ -396,10 +375,11 @@ QSet<PluginSpec *> PluginManager::pluginsRequiredByPlugin(PluginSpec *spec)
 {
     QSet<PluginSpec *> recursiveDependencies;
     recursiveDependencies.insert(spec);
-    QList<PluginSpec *> queue;
-    queue.append(spec);
-    while (!queue.isEmpty()) {
-        PluginSpec *checkSpec = queue.takeFirst();
+    std::queue<PluginSpec *> queue;
+    queue.push(spec);
+    while (!queue.empty()) {
+        PluginSpec *checkSpec = queue.front();
+        queue.pop();
         QHashIterator<PluginDependency, PluginSpec *> depIt(checkSpec->dependencySpecs());
         while (depIt.hasNext()) {
             depIt.next();
@@ -408,7 +388,7 @@ QSet<PluginSpec *> PluginManager::pluginsRequiredByPlugin(PluginSpec *spec)
             PluginSpec *depSpec = depIt.value();
             if (!recursiveDependencies.contains(depSpec)) {
                 recursiveDependencies.insert(depSpec);
-                queue.append(depSpec);
+                queue.push(depSpec);
             }
         }
     }
@@ -439,7 +419,7 @@ QString PluginManager::systemInformation() const
     if (response.result == SynchronousProcessResponse::Finished)
         result += response.allOutput() + "\n";
     result += "Plugin information:\n\n";
-    auto longestSpec = std::max_element(plugins().cbegin(), plugins().cend(),
+    auto longestSpec = std::max_element(d->pluginSpecs.cbegin(), d->pluginSpecs.cend(),
                                         [](const PluginSpec *left, const PluginSpec *right) {
                                             return left->name().size() < right->name().size();
                                         });
@@ -556,12 +536,12 @@ QStringList PluginManager::arguments()
 
     \sa setPluginPaths()
 */
-const QList<PluginSpec *> PluginManager::plugins()
+const QVector<PluginSpec *> PluginManager::plugins()
 {
     return d->pluginSpecs;
 }
 
-QHash<QString, QList<PluginSpec *> > PluginManager::pluginCollections()
+QHash<QString, QVector<PluginSpec *>> PluginManager::pluginCollections()
 {
     return d->pluginCategories;
 }
@@ -645,7 +625,7 @@ void PluginManager::remoteArguments(const QString &serializedArgument, QObject *
                                                                 arguments);
             if (socketParent && socket) {
                 socket->setParent(socketParent);
-                socket = 0;
+                socket = nullptr;
             }
         }
     }
@@ -772,7 +752,7 @@ void PluginManager::formatPluginVersions(QTextStream &str)
  */
 bool PluginManager::testRunRequested()
 {
-    return !d->testSpecs.isEmpty();
+    return !d->testSpecs.empty();
 }
 
 /*!
@@ -789,7 +769,7 @@ void PluginManager::profilingReport(const char *what, const PluginSpec *spec)
 /*!
     Returns a list of plugins in load order.
 */
-QList<PluginSpec *> PluginManager::loadQueue()
+QVector<PluginSpec *> PluginManager::loadQueue()
 {
     return d->loadQueue();
 }
@@ -838,18 +818,19 @@ PluginSpecPrivate *PluginManagerPrivate::privateSpec(PluginSpec *spec)
 
 void PluginManagerPrivate::nextDelayedInitialize()
 {
-    while (!delayedInitializeQueue.isEmpty()) {
-        PluginSpec *spec = delayedInitializeQueue.takeFirst();
+    while (!delayedInitializeQueue.empty()) {
+        PluginSpec *spec = delayedInitializeQueue.front();
+        delayedInitializeQueue.pop();
         profilingReport(">delayedInitialize", spec);
         bool delay = spec->d->delayedInitialize();
         profilingReport("<delayedInitialize", spec);
         if (delay)
             break; // do next delayedInitialize after a delay
     }
-    if (delayedInitializeQueue.isEmpty()) {
+    if (delayedInitializeQueue.empty()) {
         m_isInitializationDone = true;
         delete delayedInitializeTimer;
-        delayedInitializeTimer = 0;
+        delayedInitializeTimer = nullptr;
         profilingSummary();
         emit q->initializationDone();
 #ifdef WITH_TESTS
@@ -865,12 +846,6 @@ void PluginManagerPrivate::nextDelayedInitialize()
     \internal
 */
 PluginManagerPrivate::PluginManagerPrivate(PluginManager *pluginManager) :
-    delayedInitializeTimer(0),
-    shutdownEventLoop(0),
-    m_profileElapsedMS(0),
-    m_profilingVerbosity(0),
-    settings(0),
-    globalSettings(0),
     q(pluginManager)
 {
 }
@@ -927,9 +902,9 @@ void PluginManagerPrivate::stopAll()
     if (delayedInitializeTimer && delayedInitializeTimer->isActive()) {
         delayedInitializeTimer->stop();
         delete delayedInitializeTimer;
-        delayedInitializeTimer = 0;
+        delayedInitializeTimer = nullptr;
     }
-    QList<PluginSpec *> queue = loadQueue();
+    QVector<PluginSpec *> queue = loadQueue();
     foreach (PluginSpec *spec, queue) {
         loadPlugin(spec, PluginSpec::Stopped);
     }
@@ -947,13 +922,15 @@ void PluginManagerPrivate::deleteAll()
 
 #ifdef WITH_TESTS
 
-typedef QMap<QObject *, QStringList> TestPlan; // Object -> selected test functions
-typedef QMapIterator<QObject *, QStringList> TestPlanIterator;
+using TestPlan = QMap<QObject *, QStringList>; // Object -> selected test functions
+using TestPlanIterator = QMapIterator<QObject *, QStringList>;
 
 static bool isTestFunction(const QMetaMethod &metaMethod)
 {
-    static const QList<QByteArray> blackList = QList<QByteArray>()
-        << "initTestCase()" << "cleanupTestCase()" << "init()" << "cleanup()";
+    static const QVector<QByteArray> blackList = {"initTestCase()",
+                                                  "cleanupTestCase()",
+                                                  "init()",
+                                                  "cleanup()"};
 
     if (metaMethod.methodType() != QMetaMethod::Slot)
         return false;
@@ -1017,9 +994,9 @@ static QStringList matchingTestFunctions(const QStringList &testFunctions,
     return matchingFunctions;
 }
 
-static QObject *objectWithClassName(const QList<QObject *> &objects, const QString &className)
+static QObject *objectWithClassName(const QVector<QObject *> &objects, const QString &className)
 {
-    return Utils::findOr(objects, 0, [className] (QObject *object) -> bool {
+    return Utils::findOr(objects, nullptr, [className] (QObject *object) -> bool {
         QString candidate = QString::fromUtf8(object->metaObject()->className());
         const int colonIndex = candidate.lastIndexOf(QLatin1Char(':'));
         if (colonIndex != -1 && colonIndex < candidate.size() - 1)
@@ -1060,7 +1037,7 @@ static int executeTestPlan(const TestPlan &testPlan)
 
 /// Resulting plan consists of all test functions of the plugin object and
 /// all test functions of all test objects of the plugin.
-static TestPlan generateCompleteTestPlan(IPlugin *plugin, const QList<QObject *> &testObjects)
+static TestPlan generateCompleteTestPlan(IPlugin *plugin, const QVector<QObject *> &testObjects)
 {
     TestPlan testPlan;
 
@@ -1080,7 +1057,8 @@ static TestPlan generateCompleteTestPlan(IPlugin *plugin, const QList<QObject *>
 ///
 /// Since multiple match texts can match the same function, a test function might
 /// be included multiple times for a test object.
-static TestPlan generateCustomTestPlan(IPlugin *plugin, const QList<QObject *> &testObjects,
+static TestPlan generateCustomTestPlan(IPlugin *plugin,
+                                       const QVector<QObject *> &testObjects,
                                        const QStringList &matchTexts)
 {
     TestPlan testPlan;
@@ -1088,7 +1066,7 @@ static TestPlan generateCustomTestPlan(IPlugin *plugin, const QList<QObject *> &
     const QStringList testFunctionsOfPluginObject = testFunctions(plugin->metaObject());
     QStringList matchedTestFunctionsOfPluginObject;
     QStringList remainingMatchTexts = matchTexts;
-    QList<QObject *> remainingTestObjectsOfPlugin = testObjects;
+    QVector<QObject *> remainingTestObjectsOfPlugin = testObjects;
 
     while (!remainingMatchTexts.isEmpty()) {
         const QString matchText = remainingMatchTexts.takeFirst();
@@ -1153,11 +1131,12 @@ void PluginManagerPrivate::startTests()
         if (!plugin)
             continue; // plugin not loaded
 
-        const QList<QObject *> testObjects = plugin->createTestObjects();
+        const QVector<QObject *> testObjects = plugin->createTestObjects();
         ExecuteOnDestruction deleteTestObjects([&]() { qDeleteAll(testObjects); });
         Q_UNUSED(deleteTestObjects)
 
-        const bool hasDuplicateTestObjects = testObjects.size() != testObjects.toSet().size();
+        const bool hasDuplicateTestObjects = testObjects.size()
+                                             != Utils::filteredUnique(testObjects).size();
         QTC_ASSERT(!hasDuplicateTestObjects, continue);
         QTC_ASSERT(!testObjects.contains(plugin), continue);
 
@@ -1179,7 +1158,7 @@ void PluginManagerPrivate::addObject(QObject *obj)
 {
     {
         QWriteLocker lock(&m_lock);
-        if (obj == 0) {
+        if (obj == nullptr) {
             qWarning() << "PluginManagerPrivate::addObject(): trying to add null object";
             return;
         }
@@ -1194,7 +1173,7 @@ void PluginManagerPrivate::addObject(QObject *obj)
         if (m_profilingVerbosity && !m_profileTimer.isNull()) {
             // Report a timestamp when adding an object. Useful for profiling
             // its initialization time.
-            const int absoluteElapsedMS = m_profileTimer->elapsed();
+            const int absoluteElapsedMS = int(m_profileTimer->elapsed());
             qDebug("  %-43s %8dms", obj->metaObject()->className(), absoluteElapsedMS);
         }
 
@@ -1208,7 +1187,7 @@ void PluginManagerPrivate::addObject(QObject *obj)
 */
 void PluginManagerPrivate::removeObject(QObject *obj)
 {
-    if (obj == 0) {
+    if (obj == nullptr) {
         qWarning() << "PluginManagerPrivate::removeObject(): trying to remove null object";
         return;
     }
@@ -1231,23 +1210,27 @@ void PluginManagerPrivate::removeObject(QObject *obj)
 */
 void PluginManagerPrivate::loadPlugins()
 {
-    QList<PluginSpec *> queue = loadQueue();
+    QVector<PluginSpec *> queue = loadQueue();
+    Utils::setMimeStartupPhase(MimeStartupPhase::PluginsLoading);
     foreach (PluginSpec *spec, queue) {
         loadPlugin(spec, PluginSpec::Loaded);
     }
+    Utils::setMimeStartupPhase(MimeStartupPhase::PluginsInitializing);
     foreach (PluginSpec *spec, queue) {
         loadPlugin(spec, PluginSpec::Initialized);
     }
+    Utils::setMimeStartupPhase(MimeStartupPhase::PluginsDelayedInitializing);
     Utils::reverseForeach(queue, [this](PluginSpec *spec) {
         loadPlugin(spec, PluginSpec::Running);
         if (spec->state() == PluginSpec::Running) {
-            delayedInitializeQueue.append(spec);
+            delayedInitializeQueue.push(spec);
         } else {
             // Plugin initialization failed, so cleanup after it
             spec->d->kill();
         }
     });
     emit q->pluginsChanged();
+    Utils::setMimeStartupPhase(MimeStartupPhase::UpAndRunning);
 
     delayedInitializeTimer = new QTimer;
     delayedInitializeTimer->setInterval(DELAYED_INITIALIZE_INTERVAL);
@@ -1281,9 +1264,9 @@ void PluginManagerPrivate::shutdown()
 */
 void PluginManagerPrivate::asyncShutdownFinished()
 {
-    IPlugin *plugin = qobject_cast<IPlugin *>(sender());
+    auto *plugin = qobject_cast<IPlugin *>(sender());
     Q_ASSERT(plugin);
-    asynchronousPlugins.removeAll(plugin->pluginSpec());
+    asynchronousPlugins.remove(plugin->pluginSpec());
     if (asynchronousPlugins.isEmpty())
         shutdownEventLoop->exit();
 }
@@ -1291,11 +1274,11 @@ void PluginManagerPrivate::asyncShutdownFinished()
 /*!
     \internal
 */
-QList<PluginSpec *> PluginManagerPrivate::loadQueue()
+QVector<PluginSpec *> PluginManagerPrivate::loadQueue()
 {
-    QList<PluginSpec *> queue;
+    QVector<PluginSpec *> queue;
     foreach (PluginSpec *spec, pluginSpecs) {
-        QList<PluginSpec *> circularityCheckQueue;
+        QVector<PluginSpec *> circularityCheckQueue;
         loadQueue(spec, queue, circularityCheckQueue);
     }
     return queue;
@@ -1304,8 +1287,9 @@ QList<PluginSpec *> PluginManagerPrivate::loadQueue()
 /*!
     \internal
 */
-bool PluginManagerPrivate::loadQueue(PluginSpec *spec, QList<PluginSpec *> &queue,
-        QList<PluginSpec *> &circularityCheckQueue)
+bool PluginManagerPrivate::loadQueue(PluginSpec *spec,
+                                     QVector<PluginSpec *> &queue,
+                                     QVector<PluginSpec *> &circularityCheckQueue)
 {
     if (queue.contains(spec))
         return true;
@@ -1316,11 +1300,11 @@ bool PluginManagerPrivate::loadQueue(PluginSpec *spec, QList<PluginSpec *> &queu
         spec->d->errorString += QLatin1Char('\n');
         int index = circularityCheckQueue.indexOf(spec);
         for (int i = index; i < circularityCheckQueue.size(); ++i) {
-            spec->d->errorString.append(PluginManager::tr("%1(%2) depends on")
+            spec->d->errorString.append(PluginManager::tr("%1 (%2) depends on")
                 .arg(circularityCheckQueue.at(i)->name()).arg(circularityCheckQueue.at(i)->version()));
             spec->d->errorString += QLatin1Char('\n');
         }
-        spec->d->errorString.append(PluginManager::tr("%1(%2)").arg(spec->name()).arg(spec->version()));
+        spec->d->errorString.append(PluginManager::tr("%1 (%2)").arg(spec->name()).arg(spec->version()));
         return false;
     }
     circularityCheckQueue.append(spec);
@@ -1342,7 +1326,7 @@ bool PluginManagerPrivate::loadQueue(PluginSpec *spec, QList<PluginSpec *> &queu
         if (!loadQueue(depSpec, queue, circularityCheckQueue)) {
             spec->d->hasError = true;
             spec->d->errorString =
-                PluginManager::tr("Cannot load plugin because dependency failed to load: %1(%2)\nReason: %3")
+                PluginManager::tr("Cannot load plugin because dependency failed to load: %1 (%2)\nReason: %3")
                     .arg(depSpec->name()).arg(depSpec->version()).arg(depSpec->errorString());
             return false;
         }
@@ -1455,10 +1439,10 @@ void PluginManagerPrivate::readPluginPaths()
     pluginCategories.clear();
 
     // default
-    pluginCategories.insert(QString(), QList<PluginSpec *>());
+    pluginCategories.insert(QString(), QVector<PluginSpec *>());
 
     foreach (const QString &pluginFile, pluginFiles(pluginPaths)) {
-        PluginSpec *spec = new PluginSpec;
+        auto *spec = new PluginSpec;
         if (!spec->d->read(pluginFile)) { // not a Qt Creator plugin
             delete spec;
             continue;
@@ -1482,6 +1466,7 @@ void PluginManagerPrivate::readPluginPaths()
         pluginSpecs.append(spec);
     }
     resolveDependencies();
+    enableDependenciesIndirectly();
     // ensure deterministic plugin load order by sorting
     Utils::sort(pluginSpecs, &PluginSpec::name);
     emit q->pluginsChanged();
@@ -1489,42 +1474,19 @@ void PluginManagerPrivate::readPluginPaths()
 
 void PluginManagerPrivate::resolveDependencies()
 {
-    foreach (PluginSpec *spec, pluginSpecs) {
-        spec->d->enabledIndirectly = false; // reset, is recalculated below
+    foreach (PluginSpec *spec, pluginSpecs)
         spec->d->resolveDependencies(pluginSpecs);
-    }
-
-    Utils::reverseForeach(loadQueue(), [](PluginSpec *spec) {
-        spec->d->enableDependenciesIndirectly();
-    });
 }
 
-void PluginManagerPrivate::enableOnlyTestedSpecs()
+void PluginManagerPrivate::enableDependenciesIndirectly()
 {
-    if (testSpecs.isEmpty())
-        return;
-
-    QList<PluginSpec *> specsForTests;
-    foreach (const TestSpec &testSpec, testSpecs) {
-        QList<PluginSpec *> circularityCheckQueue;
-        loadQueue(testSpec.pluginSpec, specsForTests, circularityCheckQueue);
-        // add plugins that must be force loaded when running tests for the plugin
-        // (aka "test dependencies")
-        QHashIterator<PluginDependency, PluginSpec *> it(testSpec.pluginSpec->dependencySpecs());
-        while (it.hasNext()) {
-            it.next();
-            if (it.key().type != PluginDependency::Test)
-                continue;
-            PluginSpec *depSpec = it.value();
-            circularityCheckQueue.clear();
-            loadQueue(depSpec, specsForTests, circularityCheckQueue);
-        }
-    }
     foreach (PluginSpec *spec, pluginSpecs)
-        spec->d->setForceDisabled(true);
-    foreach (PluginSpec *spec, specsForTests) {
-        spec->d->setForceDisabled(false);
-        spec->d->setForceEnabled(true);
+        spec->d->enabledIndirectly = false;
+    // cannot use reverse loadQueue here, because test dependencies can introduce circles
+    QVector<PluginSpec *> queue = Utils::filtered(pluginSpecs, &PluginSpec::isEffectivelyEnabled);
+    while (!queue.isEmpty()) {
+        PluginSpec *spec = queue.takeFirst();
+        queue += spec->d->enableDependenciesIndirectly(containsTestSpec(spec));
     }
 }
 
@@ -1543,7 +1505,7 @@ PluginSpec *PluginManagerPrivate::pluginForOption(const QString &option, bool *r
             return spec;
         }
     }
-    return 0;
+    return nullptr;
 }
 
 PluginSpec *PluginManagerPrivate::pluginByName(const QString &name) const
@@ -1554,7 +1516,7 @@ PluginSpec *PluginManagerPrivate::pluginByName(const QString &name) const
 void PluginManagerPrivate::initProfiling()
 {
     if (m_profileTimer.isNull()) {
-        m_profileTimer.reset(new QTime);
+        m_profileTimer.reset(new QElapsedTimer);
         m_profileTimer->start();
         m_profileElapsedMS = 0;
         qDebug("Profiling started");
@@ -1566,15 +1528,22 @@ void PluginManagerPrivate::initProfiling()
 void PluginManagerPrivate::profilingReport(const char *what, const PluginSpec *spec /* = 0 */)
 {
     if (!m_profileTimer.isNull()) {
-        const int absoluteElapsedMS = m_profileTimer->elapsed();
+        const int absoluteElapsedMS = int(m_profileTimer->elapsed());
         const int elapsedMS = absoluteElapsedMS - m_profileElapsedMS;
         m_profileElapsedMS = absoluteElapsedMS;
-        if (spec)
-            m_profileTotal[spec] += elapsedMS;
         if (spec)
             qDebug("%-22s %-22s %8dms (%8dms)", what, qPrintable(spec->name()), absoluteElapsedMS, elapsedMS);
         else
             qDebug("%-45s %8dms (%8dms)", what, absoluteElapsedMS, elapsedMS);
+        if (what && *what == '<') {
+            QString tc;
+            if (spec) {
+                m_profileTotal[spec] += elapsedMS;
+                tc = spec->name() + '_';
+            }
+            tc += QString::fromUtf8(QByteArray(what + 1));
+            Utils::Benchmarker::report("loadPlugins", tc, elapsedMS);
+        }
     }
 }
 
@@ -1595,75 +1564,24 @@ void PluginManagerPrivate::profilingSummary() const
             qDebug("%-22s %8dms   ( %5.2f%% )", qPrintable(it.value()->name()),
                 it.key(), 100.0 * it.key() / total);
          qDebug("Total: %8dms", total);
+         Utils::Benchmarker::report("loadPlugins", "Total", total);
     }
 }
 
 static inline QString getPlatformName()
 {
-    if (HostOsInfo::isMacHost()) {
-        if (QSysInfo::MacintoshVersion >= QSysInfo::MV_10_0) {
-            QString result = QLatin1String("OS X");
-            result += QLatin1String(" 10.") + QString::number(QSysInfo::MacintoshVersion - QSysInfo::MV_10_0);
-            return result;
-        } else {
-            return QLatin1String("Mac OS");
-        }
-    } else if (HostOsInfo::isAnyUnixHost()) {
-        QString base = QLatin1String(HostOsInfo::isLinuxHost() ? "Linux" : "Unix");
-        QFile osReleaseFile(QLatin1String("/etc/os-release")); // Newer Linuxes
-        if (osReleaseFile.open(QIODevice::ReadOnly)) {
-            QString name;
-            QString version;
-            forever {
-                const QByteArray line = osReleaseFile.readLine();
-                if (line.isEmpty())
-                    break;
-                if (line.startsWith("NAME=\""))
-                    name = QString::fromLatin1(line.mid(6, line.size() - 8)).trimmed();
-                if (line.startsWith("VERSION_ID=\""))
-                    version = QString::fromLatin1(line.mid(12, line.size() - 14)).trimmed();
-            }
-            if (!name.isEmpty()) {
-                if (!version.isEmpty())
-                    name += QLatin1Char(' ') + version;
-                return base + QLatin1String(" (") + name + QLatin1Char(')');
-            }
-        }
-        return base;
-    } else if (HostOsInfo::isWindowsHost()) {
-        QString result = QLatin1String("Windows");
-        switch (QSysInfo::WindowsVersion) {
-        case QSysInfo::WV_XP:
-            result += QLatin1String(" XP");
-            break;
-        case QSysInfo::WV_2003:
-            result += QLatin1String(" 2003");
-            break;
-        case QSysInfo::WV_VISTA:
-            result += QLatin1String(" Vista");
-            break;
-        case QSysInfo::WV_WINDOWS7:
-            result += QLatin1String(" 7");
-            break;
-        case QSysInfo::WV_WINDOWS8:
-            result += QLatin1String(" 8");
-            break;
-        case QSysInfo::WV_WINDOWS8_1:
-            result += QLatin1String(" 8.1");
-            break;
-        default:
-            break;
-        }
-        if (QSysInfo::WindowsVersion >= QSysInfo::WV_WINDOWS10)
-            result += QLatin1String(" 10");
-        return result;
-    }
+    if (HostOsInfo::isMacHost())
+        return QLatin1String("OS X");
+    else if (HostOsInfo::isAnyUnixHost())
+        return QLatin1String(HostOsInfo::isLinuxHost() ? "Linux" : "Unix");
+    else if (HostOsInfo::isWindowsHost())
+        return QLatin1String("Windows");
     return QLatin1String("Unknown");
 }
 
 QString PluginManager::platformName()
 {
-    static const QString result = getPlatformName();
+    static const QString result = getPlatformName() + " (" + QSysInfo::prettyProductName() + ')';
     return result;
 }
 
@@ -1685,17 +1603,4 @@ QObject *PluginManager::getObjectByName(const QString &name)
     });
 }
 
-/*!
-    Retrieves one object inheriting a class with \a className from the object
-    pool.
-    \sa addObject()
-*/
-
-QObject *PluginManager::getObjectByClassName(const QString &className)
-{
-    const QByteArray ba = className.toUtf8();
-    QReadLocker lock(&d->m_lock);
-    return Utils::findOrDefault(allObjects(), [&ba](const QObject *obj) {
-        return obj->inherits(ba.constData());
-    });
-}
+} // ExtensionSystem
