@@ -27,6 +27,8 @@
 #include "testresult.h"
 #include "testresultspane.h"
 
+#include <utils/qtcassert.h>
+
 #include <QDebug>
 #include <QProcess>
 
@@ -38,42 +40,62 @@ TestOutputReader::TestOutputReader(const QFutureInterface<TestResultPtr> &future
     : m_futureInterface(futureInterface)
     , m_testApplication(testApplication)
     , m_buildDir(buildDirectory)
+    , m_id(testApplication ? testApplication->program() : QString())
 {
     if (m_testApplication) {
         connect(m_testApplication, &QProcess::readyRead,
                 this, [this] () {
             while (m_testApplication->canReadLine()) {
-                QByteArray output = m_testApplication->readLine();
-                output.chop(1); // remove the newline from the output
-                if (output.endsWith('\r'))
-                    output.chop(1);
-
-                emit newOutputAvailable(output);
+                const QByteArray output = m_testApplication->readLine();
                 processOutput(output);
             }
         });
         connect(m_testApplication, &QProcess::readyReadStandardError,
                 this, [this] () {
             const QByteArray output = m_testApplication->readAllStandardError();
-            emit newOutputAvailable(output);
             processStdError(output);
         });
     }
 }
 
-void TestOutputReader::processStdError(const QByteArray &output)
+void TestOutputReader::processOutput(const QByteArray &output)
 {
-    qWarning() << "AutoTest.Run: Ignored plain output:" << output;
+    processOutputLine(output);
+    emit newOutputAvailable(output);
+}
+
+void TestOutputReader::processStdError(const QByteArray &outputLineWithNewLine)
+{
+    qWarning() << "AutoTest.Run: Ignored plain output:" << outputLineWithNewLine;
+    emit newOutputAvailable(outputLineWithNewLine);
 }
 
 void TestOutputReader::reportCrash()
 {
     TestResultPtr result = createDefaultResult();
     result->setDescription(tr("Test executable crashed."));
-    result->setResult(Result::MessageFatal);
+    result->setResult(ResultType::MessageFatal);
     m_futureInterface.reportResult(result);
 }
 
+void TestOutputReader::createAndReportResult(const QString &message, ResultType type)
+{
+    TestResultPtr result = createDefaultResult();
+    result->setDescription(message);
+    result->setResult(type);
+    reportResult(result);
+}
+
+QByteArray TestOutputReader::chopLineBreak(const QByteArray &original)
+{
+    QTC_ASSERT(original.endsWith('\n'), return original);
+    QByteArray output(original);
+    output.chop(1); // remove the newline from the output
+    if (output.endsWith('\r'))
+        output.chop(1);
+    return output;
+}
+
 void TestOutputReader::reportResult(const TestResultPtr &result)
 {
     m_futureInterface.reportResult(result);
