@@ -62,6 +62,7 @@
 #include <projectexplorer/projectmacro.h>
 #include <projectexplorer/session.h>
 #include <utils/fileutils.h>
+#include <utils/hostosinfo.h>
 #include <utils/qtcassert.h>
 
 #include <QCoreApplication>
@@ -146,7 +147,7 @@ public:
     mutable QMutex m_projectMutex;
     QMap<ProjectExplorer::Project *, ProjectInfo> m_projectToProjectsInfo;
     QHash<ProjectExplorer::Project *, bool> m_projectToIndexerCanceled;
-    QMap<Utils::FileName, QList<ProjectPart::Ptr> > m_fileToProjectParts;
+    QMap<Utils::FilePath, QList<ProjectPart::Ptr> > m_fileToProjectParts;
     QMap<QString, ProjectPart::Ptr> m_projectPartIdToProjectProjectPart;
     // The members below are cached/(re)calculated from the projects and/or their project parts
     bool m_dirty;
@@ -478,7 +479,7 @@ void CppModelManager::initCppTools()
             this, &CppModelManager::updateModifiedSourceFiles);
     connect(Core::DocumentManager::instance(), &Core::DocumentManager::filesChangedInternally,
             [this](const QStringList &files) {
-        updateSourceFiles(files.toSet());
+        updateSourceFiles(Utils::toSet(files));
     });
 
     connect(this, &CppModelManager::documentUpdated,
@@ -513,6 +514,10 @@ CppModelManager::CppModelManager()
     d->m_indexingSupporter = nullptr;
     d->m_enableGC = true;
 
+    // Visual C++ has 1MiB, macOSX has 512KiB
+    if (Utils::HostOsInfo::isWindowsHost() || Utils::HostOsInfo::isMacHost())
+        d->m_threadPool.setStackSize(2 * 1024 * 1024);
+
     qRegisterMetaType<QSet<QString> >();
     connect(this, &CppModelManager::sourceFilesRefreshed,
             this, &CppModelManager::onSourceFilesRefreshed);
@@ -920,7 +925,7 @@ public:
     {
         QSet<QString> removed = projectPartIds(m_old.projectParts());
         removed.subtract(projectPartIds(m_new.projectParts()));
-        return removed.toList();
+        return Utils::toList(removed);
     }
 
     /// Returns a list of common files that have a changed timestamp.
@@ -968,7 +973,7 @@ void CppModelManager::recalculateProjectPartMappings()
         foreach (const ProjectPart::Ptr &projectPart, projectInfo.projectParts()) {
             d->m_projectPartIdToProjectProjectPart[projectPart->id()] = projectPart;
             foreach (const ProjectFile &cxxFile, projectPart->files)
-                d->m_fileToProjectParts[Utils::FileName::fromString(cxxFile.path)].append(
+                d->m_fileToProjectParts[Utils::FilePath::fromString(cxxFile.path)].append(
                             projectPart);
 
         }
@@ -1015,7 +1020,7 @@ void CppModelManager::updateCppEditorDocuments(bool projectsUpdated) const
 
     // Mark invisible documents dirty
     QSet<Core::IDocument *> invisibleCppEditorDocuments
-        = Core::DocumentModel::openedDocuments().toSet();
+        = Utils::toSet(Core::DocumentModel::openedDocuments());
     invisibleCppEditorDocuments.subtract(visibleCppEditorDocuments);
     foreach (Core::IDocument *document, invisibleCppEditorDocuments) {
         const QString filePath = document->filePath().toString();
@@ -1080,7 +1085,7 @@ QFuture<void> CppModelManager::updateProjectInfo(QFutureInterface<void> &futureI
                 const QSet<QString> removedFiles = comparer.removedFiles();
                 if (!removedFiles.isEmpty()) {
                     filesRemoved = true;
-                    emit aboutToRemoveFiles(removedFiles.toList());
+                    emit aboutToRemoveFiles(Utils::toList(removedFiles));
                     removeFilesFromSnapshot(removedFiles);
                 }
             }
@@ -1132,22 +1137,21 @@ ProjectPart::Ptr CppModelManager::projectPartForId(const QString &projectPartId)
     return d->m_projectPartIdToProjectProjectPart.value(projectPartId);
 }
 
-QList<ProjectPart::Ptr> CppModelManager::projectPart(const Utils::FileName &fileName) const
+QList<ProjectPart::Ptr> CppModelManager::projectPart(const Utils::FilePath &fileName) const
 {
     QMutexLocker locker(&d->m_projectMutex);
     return d->m_fileToProjectParts.value(fileName);
 }
 
 QList<ProjectPart::Ptr> CppModelManager::projectPartFromDependencies(
-        const Utils::FileName &fileName) const
+        const Utils::FilePath &fileName) const
 {
     QSet<ProjectPart::Ptr> parts;
-    const Utils::FileNameList deps = snapshot().filesDependingOn(fileName);
+    const Utils::FilePathList deps = snapshot().filesDependingOn(fileName);
 
     QMutexLocker locker(&d->m_projectMutex);
-    foreach (const Utils::FileName &dep, deps) {
-        parts.unite(QSet<ProjectPart::Ptr>::fromList(d->m_fileToProjectParts.value(dep)));
-    }
+    for (const Utils::FilePath &dep : deps)
+        parts.unite(Utils::toSet(d->m_fileToProjectParts.value(dep)));
 
     return parts.values();
 }
@@ -1213,10 +1217,10 @@ void CppModelManager::delayedGC()
 
 static QStringList removedProjectParts(const QStringList &before, const QStringList &after)
 {
-    QSet<QString> b = before.toSet();
-    b.subtract(after.toSet());
+    QSet<QString> b = Utils::toSet(before);
+    b.subtract(Utils::toSet(after));
 
-    return b.toList();
+    return Utils::toList(b);
 }
 
 void CppModelManager::onAboutToRemoveProject(ProjectExplorer::Project *project)
@@ -1338,7 +1342,7 @@ void CppModelManager::GC()
         filesInEditorSupports << abstractEditorSupport->fileName();
 
     Snapshot currentSnapshot = snapshot();
-    QSet<Utils::FileName> reachableFiles;
+    QSet<Utils::FilePath> reachableFiles;
     // The configuration file is part of the project files, which is just fine.
     // If single files are open, without any project, then there is no need to
     // keep the configuration file around.
@@ -1349,7 +1353,7 @@ void CppModelManager::GC()
         const QString file = todo.last();
         todo.removeLast();
 
-        const Utils::FileName fileName = Utils::FileName::fromString(file);
+        const Utils::FilePath fileName = Utils::FilePath::fromString(file);
         if (reachableFiles.contains(fileName))
             continue;
         reachableFiles.insert(fileName);
@@ -1362,7 +1366,7 @@ void CppModelManager::GC()
     QStringList notReachableFiles;
     Snapshot newSnapshot;
     for (Snapshot::const_iterator it = currentSnapshot.begin(); it != currentSnapshot.end(); ++it) {
-        const Utils::FileName &fileName = it.key();
+        const Utils::FilePath &fileName = it.key();
 
         if (reachableFiles.contains(fileName))
             newSnapshot.insert(it.value());
