@@ -62,6 +62,7 @@ DocumentWarningWidget::DocumentWarningWidget(QWidget *parent)
     m_headerLabel->setFont(boldFont);
     m_messageLabel->setForegroundRole(QPalette::ToolTipText);
     m_messageLabel->setWordWrap(true);
+    m_messageLabel->setTextInteractionFlags(Qt::TextSelectableByMouse);
 
     m_ignoreWarningsCheckBox->setText(tr("Always ignore these warnings about features "
                                          "not supported by Qt Quick Designer."));
@@ -87,16 +88,16 @@ DocumentWarningWidget::DocumentWarningWidget(QWidget *parent)
 
     connect(m_ignoreWarningsCheckBox, &QCheckBox::toggled, this, &DocumentWarningWidget::ignoreCheckBoxToggled);
 
-    QVBoxLayout *layout = new QVBoxLayout(this);
+    auto layout = new QVBoxLayout(this);
     layout->addWidget(m_headerLabel);
-    QVBoxLayout *messageLayout = new QVBoxLayout;
+    auto messageLayout = new QVBoxLayout;
     messageLayout->setMargin(20);
     messageLayout->setSpacing(5);
     messageLayout->addWidget(m_navigateLabel);
     messageLayout->addWidget(m_messageLabel);
     layout->addLayout(messageLayout);
     layout->addWidget(m_ignoreWarningsCheckBox);
-    QHBoxLayout *buttonLayout = new QHBoxLayout();
+    auto buttonLayout = new QHBoxLayout();
     buttonLayout->addStretch();
     buttonLayout->addWidget(m_continueButton);
     layout->addLayout(buttonLayout);
