@@ -25,6 +25,7 @@
 
 #include "keywordscompletionassist.h"
 
+#include <coreplugin/coreconstants.h>
 #include <texteditor/codeassist/assistinterface.h>
 #include <texteditor/codeassist/genericproposal.h>
 #include <texteditor/codeassist/functionhintproposal.h>
@@ -34,7 +35,11 @@
 #include <texteditor/texteditorconstants.h>
 #include <texteditor/texteditor.h>
 
+#include <QDir>
+#include <QFileInfo>
+
 #include <utils/algorithm.h>
+#include <utils/utilsicons.h>
 
 namespace TextEditor {
 
@@ -190,10 +195,14 @@ IAssistProposal *KeywordsCompletionAssistProcessor::perform(const AssistInterfac
 
     if (interface->reason() == IdleEditor) {
         QChar characterUnderCursor = interface->characterAt(interface->position());
-        if (characterUnderCursor.isLetterOrNumber())
-            return nullptr;
-        if (interface->position() - startPosition < 3)
-            return nullptr;
+        if (characterUnderCursor.isLetterOrNumber() || interface->position() - startPosition < 3) {
+            QList<AssistProposalItemInterface *> items;
+            if (m_dynamicCompletionFunction)
+                m_dynamicCompletionFunction(interface, &items, startPosition);
+            if (items.isEmpty())
+                return nullptr;
+            return new GenericProposal(startPosition, items);
+        }
     }
 
     // extract word
@@ -210,9 +219,15 @@ IAssistProposal *KeywordsCompletionAssistProcessor::perform(const AssistInterfac
         FunctionHintProposalModelPtr model(new KeywordsFunctionHintModel(functionSymbols));
         return new FunctionHintProposal(startPosition, model);
     } else {
-        QList<AssistProposalItemInterface *> items = m_snippetCollector.collect();
-        items.append(generateProposalList(m_keywords.variables(), m_variableIcon));
-        items.append(generateProposalList(m_keywords.functions(), m_functionIcon));
+        const int originalStartPos = startPosition;
+        QList<AssistProposalItemInterface *> items;
+        if (m_dynamicCompletionFunction)
+            m_dynamicCompletionFunction(interface, &items, startPosition);
+        if (startPosition == originalStartPos) {
+            items.append(m_snippetCollector.collect());
+            items.append(generateProposalList(m_keywords.variables(), m_variableIcon));
+            items.append(generateProposalList(m_keywords.functions(), m_functionIcon));
+        }
         return new GenericProposal(startPosition, items);
     }
 }
@@ -227,6 +242,11 @@ void KeywordsCompletionAssistProcessor::setKeywords(const Keywords &keywords)
     m_keywords = keywords;
 }
 
+void KeywordsCompletionAssistProcessor::setDynamicCompletionFunction(DynamicCompletionFunction func)
+{
+    m_dynamicCompletionFunction = func;
+}
+
 bool KeywordsCompletionAssistProcessor::isInComment(const AssistInterface *interface) const
 {
     QTextCursor tc(interface->textDocument());
@@ -252,6 +272,12 @@ KeywordsCompletionAssistProvider::KeywordsCompletionAssistProvider(const Keyword
     , m_snippetGroup(snippetGroup)
 { }
 
+void KeywordsCompletionAssistProvider::setDynamicCompletionFunction(
+        const DynamicCompletionFunction &func)
+{
+    m_completionFunc = func;
+}
+
 IAssistProvider::RunType KeywordsCompletionAssistProvider::runType() const
 {
     return Synchronous;
@@ -261,7 +287,66 @@ IAssistProcessor *KeywordsCompletionAssistProvider::createProcessor() const
 {
     auto processor = new KeywordsCompletionAssistProcessor(m_keyWords);
     processor->setSnippetGroup(m_snippetGroup);
+    processor->setDynamicCompletionFunction(m_completionFunc);
     return processor;
 }
 
+void pathComplete(const AssistInterface *interface, QList<AssistProposalItemInterface *> *items,
+                  int &startPosition)
+{
+    if (!items)
+        return;
+
+    if (interface->fileName().isEmpty())
+        return;
+
+    // For pragmatic reasons, we don't support spaces in file names here.
+    static const auto canOccurInFilePath = [](const QChar &c) {
+        return c.isLetterOrNumber() || c == '.' || c == '/' || c == "_" || c == '-';
+    };
+
+    int pos = interface->position();
+    QChar chr;
+    // Skip to the start of a name
+    do {
+        chr = interface->characterAt(--pos);
+    } while (canOccurInFilePath(chr));
+
+    const int startPos= ++pos;
+
+    if (interface->reason() == IdleEditor && interface->position() - startPos < 3)
+        return;
+
+    const QString word = interface->textAt(startPos, interface->position() - startPos);
+    QDir baseDir = QFileInfo(interface->fileName()).absoluteDir();
+    const int lastSlashPos = word.lastIndexOf(QLatin1Char('/'));
+
+    QString prefix = word;
+    if (lastSlashPos != -1) {
+        prefix = word.mid(lastSlashPos +1);
+        if (!baseDir.cd(word.left(lastSlashPos)))
+            return;
+    }
+
+    const QFileInfoList entryInfoList
+            = baseDir.entryInfoList(QDir::AllEntries | QDir::NoDotAndDotDot);
+    for (const QFileInfo &entry : entryInfoList) {
+        const QString &fileName = entry.fileName();
+        if (fileName.startsWith(prefix)) {
+            AssistProposalItem *item = new AssistProposalItem;
+            if (entry.isDir()) {
+                item->setText(fileName + QLatin1String("/"));
+                item->setIcon(Utils::Icons::DIR.icon());
+            } else {
+                item->setText(fileName);
+                item->setIcon(Utils::Icons::UNKNOWN_FILE.icon());
+            }
+            *items << item;
+        }
+    }
+    if (!items->empty())
+        startPosition = startPos;
+}
+
+
 } // namespace TextEditor
